using System;
using System.Collections.Generic;
using System.Text;

namespace SequencerDemo
{
    class Recorder
    {
        List<Channel> channels;
        long start;
        public Recorder()
        {
            channels = new List<Channel>();
            start = -1;
        }
        public void NoteOn(int note)
        {
            if (start == -1)
                start = DateTime.Now.Ticks;
            long time = DateTime.Now.Ticks - start;
            foreach (Channel c in channels)
                if (c.AddNote(note, time))
                    return;
            channels.Add(new Channel());
            if (!channels[channels.Count - 1].AddNote(note, time))
                throw new Exception("Internal Error!");
        }
        public void NoteOff(int note)
        {
            long time = DateTime.Now.Ticks - start;
            foreach (Channel c in channels)
                if (c.NoteOff(note, time))
                    break;
        }
        const long viertel = 2500000;
        const long mv = 16;
        internal string GetString()
        {
            string s = "MIDI:\r\n.dw " + channels.Count + "\r\n";
            for (int i = 0; i < channels.Count; i++)
                s += ".dw (S" + i + " - SMIDI)*2\r\n";
            s += "SMIDI:\r\n";
            for (int i = 0; i < channels.Count; i++)
            {
                s += "S" + i + ":\r\n.db ";
                long bevor = 0;
                int x = 0;
                foreach (Note n in channels[i].notes)
                {
                    int delay = (int)((n.starttime - bevor) * mv * Form1.factor / viertel);
                    bevor += ((long)delay) * viertel / Form1.factor / mv;
                    s += Delay(delay);

                    delay = (int)((n.endtime - n.starttime) * mv * Form1.factor / viertel);

                    bevor += ((long)delay) * viertel / Form1.factor / mv;
                    while (delay > 256)
                    {
                        s += "TONE, " + n.note + ", 0, ";
                        delay -= 256;
                        x++;
                    }
                    if (delay != 0)
                    {
                        s += "TONE, " + n.note + ", " + delay + ", ";
                        x++;
                    }
                    if (x > 10 && x % 2 == 0)
                    {
                        s = s.Remove(s.Length - 2) + "\r\n.db ";
                        x = 0;
                    }
                }
                s += "END\r\n";
            }
            return s;
        }

        string Delay(int delay)
        {
            string s = "";
            if (delay == 0)
                return s;
            while (delay > 256)
            {
                s += "DELAY, " + 0 + ", ";
                delay -= 256;
            }
            if(delay != 0)
                s += "DELAY, " + delay + ", ";
            return s;
        }
    }

    class Channel
    {
        public Channel()
        {
            notes = new List<Note>();
        }
        bool playing;
        public List<Note> notes;
        public bool AddNote(int note, long time)
        {
            if (!playing)
            {
                notes.Add(new Note(time, (byte)note));
                playing = true;
                return true;
            }
            return false;
        }
        public bool NoteOff(int note, long time)
        {
            if (notes[notes.Count - 1].note == note)
            {
                notes[notes.Count - 1] = new Note(notes[notes.Count - 1], time);
                playing = false;
                return true;
            }
            return false;
        }
    }

    struct Note
    {
        public Note(long starttime, byte note)
        {
            this.starttime = starttime;
            this.note = note;
            endtime = 0;
        }
        public Note(Note n, long endtime)
        {
            starttime = n.starttime;
            note = n.note;
            this.endtime = endtime;
        }
        public long starttime;
        public long endtime;
        public byte note;
    }
}
